@@ -205,7 +205,6 @@ class argWordBackend(argBackendBase):
 
             # Iterate over all items in string helper
             for string, font_bits, color, highlight_color in multi_font_string.iterator():
-
                 if isinstance(font_bits, int):
                     # Map Greek letter to unicode when requested
                     if font_bits == 16:
@@ -264,19 +263,65 @@ class argWordBackend(argBackendBase):
                         run.font.name = self.fonts.get("calligraphic", "default" if self.fonts.get("default") else None)
                     elif 9 in font_bits:
                         run.font.size = Pt(14)
+                        run.font.bold = True
                     elif 10 in font_bits:
                         run.font.size = Pt(13)
+                        run.font.bold = True
                     elif 11 in font_bits:
                         run.font.size = Pt(11)
+                        run.font.bold = True
                     elif 12 in font_bits:
                         run.font.size = Pt(11)
+                        run.font.bold = True
+                        run.font.italic = True
                     elif 13 in font_bits:
                         run.font.size = Pt(11)
                     elif 14 in font_bits:
                         run.font.size = Pt(11)
+                        run.font.italic = True
                     else:
                         run.font.name = self.fonts.get("default", None)
 
+                elif isinstance(font_bits, dict):
+                    run = paragraph.add_run(string)
+                    if font_bits.get('font-size') is not None:
+                        run.font.size = Pt(font_bits.get('font-size', 11))
+                    if font_bits.get('font-family') is not None:
+                        run.font.name = font_bits.get('font-family')
+                    if font_bits.get('font-list') is not None and font_bits.get('font-list'):
+                        font_list = font_bits.get('font-list')
+                        run.font.italic = True if 1 in font_list else False
+                        run.font.bold = True if 2 in font_list else False
+                        run.font.underline = True if 3 in font_list else False
+                        run.font.strike = True if 5 in font_list else False
+                        run.font.subscript = True if 6 in font_list else False
+                        if 4 in font_list:
+                            run.font.name = self.fonts.get("typewriter",
+                                                           "default" if self.fonts.get("default") else None)
+                        elif 8 in font_list:
+                            run.font.name = self.fonts.get("calligraphic",
+                                                           "default" if self.fonts.get("default") else None)
+                        elif 9 in font_list:
+                            run.font.size = Pt(14)
+                            run.font.bold = True
+                        elif 10 in font_list:
+                            run.font.size = Pt(13)
+                            run.font.bold = True
+                        elif 11 in font_list:
+                            run.font.size = Pt(11)
+                            run.font.bold = True
+                        elif 12 in font_list:
+                            run.font.size = Pt(11)
+                            run.font.bold = True
+                            run.font.italic = True
+                        elif 13 in font_list:
+                            run.font.size = Pt(11)
+                        elif 14 in font_list:
+                            run.font.size = Pt(11)
+                            run.font.italic = True
+                        else:
+                            run.font.name = self.fonts.get("default", None)
+
                 if color:
                     if self.colors.get(color, None) is None:
                         rgb = color.split(',')
@@ -503,6 +548,8 @@ class argWordBackend(argBackendBase):
         """
 
         # Call appropriate subdivision method
+        if self.check_orientation_change(item=item):
+            self.change_orientation()
         self.add_subdivision(item, 4)
 
     def add_subsection(self, item, numbered=True):
@@ -510,6 +557,8 @@ class argWordBackend(argBackendBase):
         """
 
         # Call appropriate subdivision method
+        if self.check_orientation_change(item=item):
+            self.change_orientation()
         self.add_subdivision(item, 3)
 
     def add_section(self, item, numbered=True):
@@ -517,6 +566,8 @@ class argWordBackend(argBackendBase):
         """
 
         # Call appropriate subdivision method
+        if self.check_orientation_change(item=item):
+            self.change_orientation()
         self.add_subdivision(item, 2)
 
     def add_chapter(self, item, numbered=True):
@@ -527,6 +578,8 @@ class argWordBackend(argBackendBase):
         self.Report.add_page_break()
 
         # Call appropriate subdivision method
+        if self.check_orientation_change(item=item):
+            self.change_orientation()
         self.add_subdivision(item, 1)
 
     def add_page_break(self):
@@ -667,11 +720,28 @@ class argWordBackend(argBackendBase):
         current_section = self.Report.sections[-1]
         new_width, new_height = current_section.page_height, current_section.page_width
         new_section = self.Report.add_section(WD_SECTION_START.NEW_PAGE)
-        new_section.orientation = WD_ORIENTATION.LANDSCAPE
+        if self.Orientation == 'portrait':
+            new_section.orientation = WD_ORIENTATION.LANDSCAPE
+            self.Orientation = 'landscape'
+        elif self.Orientation == 'landscape':
+            new_section.orientation = WD_ORIENTATION.PORTRAIT
+            self.Orientation = 'portrait'
         new_section.page_width = new_width
         new_section.page_height = new_height
         return new_section
 
+    def check_orientation_change(self, item: dict) -> bool:
+        """ Returns True if the page orientation needs to be changed. Returns False if no changes needed.
+        """
+        if item.get('orientation', None) is None and self.Orientation == 'portrait':
+            return False
+        elif item.get('orientation', None) is None and self.Orientation == 'landscape':
+            return True
+        elif item.get('orientation', None) == self.Orientation:
+            return False
+        elif item.get('orientation', None) != self.Orientation:
+            return True
+
     def add_color_table(self, data: dict) -> None:
         """ Add colored table to the report
         """
@@ -777,9 +847,9 @@ class argWordBackend(argBackendBase):
             # Setting counter coresponding to paragraphs
             counter = 0
             # Iterating over elements in document to inline
-            for element in inline_docx.element.body:
+            for element in inline_docx.paragraphs:
                 # If element is a paragraph, counter gets incremented
-                if isinstance(element, docx.oxml.text.paragraph.CT_P):
+                if isinstance(element, Paragraph):
                     # Eliminate use of style which does not exist
                     if element.style == 'Caption1':
                         element.style = 'Caption'
@@ -810,52 +880,108 @@ class argWordBackend(argBackendBase):
                         counter += 1
                     # Paragraph with no photo is simply added to the current report
                     else:
-                        self.Report.element.body.append(element)
+                        if element.text:
+                            new_par = self.Report.add_paragraph()
+                            self.copy_paragraph(copy_to=new_par, copy_from=element)
                         counter += 1
-                # If element was not a paragraph is simply added to the current report
-                else:
-                    self.Report.element.body.append(element)
+
+    @staticmethod
+    def copy_paragraph(copy_to: Paragraph, copy_from: Paragraph):
+        """
+        Copies everything from `copy_from` to `copy_to` paragraph.
+        """
+        for run in copy_from.runs:
+            output_run = copy_to.add_run(run.text, run.style)
+            output_run.bold = run.bold
+            output_run.italic = run.italic
+            output_run.underline = run.underline
+            output_run.font.color.rgb = run.font.color.rgb
+            output_run.style.name = run.style.name
+        copy_to.paragraph_format.alignment = copy_from.paragraph_format.alignment
+        if copy_from.style.style_id == 'Caption1':
+            copy_from.style.style_id = 'Caption'
+            copy_from.style.style_id = 'Caption'
+            copy_to.style = copy_from.style
+        else:
+            copy_to.style = copy_from.style
 
     def add_html(self, data: dict) -> None:
         """ Adds html content to Word document. """
+        list_type_map = {'ul': 'List Bullet', 'ol': 'List Number'}
+        alingment_map = {'CENTER': WD_ALIGN_PARAGRAPH.CENTER, 'LEFT': WD_ALIGN_PARAGRAPH.LEFT,
+                         'RIGHT': WD_ALIGN_PARAGRAPH.RIGHT, 'JUSTIFY': WD_ALIGN_PARAGRAPH.JUSTIFY}
         # Reading html data as a string
         html_str = data.get('html_string', None)
         # Parsing html data and get a html document in a form of list
         arg_html_parser = ArgHTMLParser()
         arg_html_parser.reset()
         html_list = arg_html_parser.get_mapped_html(html_str)
-        # Parsing an html list into nested list over which is easier to iterate
-        nested_list = self.nesting_html_list(html_list=html_list)
-        # Parsing nested list to a list of mainly ARG Multi Font String Helper, ready to put in directly into document
-        amfsh_list = self.map_nested_list_to_amfsh(nested_list=nested_list)
+        # Parsing a html list into flat list over which is easier to iterate
+        flat_list = self.get_flat_list(html_list=html_list)
+        combined_list = self.combine_attrs(flat_list=flat_list)
+        # Parsing flat list to a list of mainly ARG Multi Font String Helper, ready to put in directly into document
+        amfsh_dict = self.map_combined_list_to_amfsh(combined_list=combined_list)
         # Final iteration over ARG Multi Font String Helper list
-        for amfsh in amfsh_list:
-            # Checking for tuple. Tuple instance is dedicated for any element except ordered and unordered lists
-            if isinstance(amfsh, tuple):
-                # amfsh as tuple: (alignment, argMultiFontStringHelper, margin)
+        # Setting up helpers for resetting ordered list numbering
+        num_res_help_ol = {num: None for num in range(1, 10)}
+        num_res_help_ul = {num: None for num in range(1, 10)}
+        for uuid_key, amfsh in amfsh_dict.get('amfsh').items():
+            if (lst := amfsh_dict.get('html_list').get(uuid_key)) and lst is not None:
+                list_type = [le.get('symbol') for le in lst if le.get('symbol') in list_type_map][-1]
+                list_level = [le.get('list_level') for le in lst if le.get('symbol') in list_type_map][-1]
+                par_style = f"{list_type_map.get(list_type)}" if list_level == 1 else \
+                    f"{list_type_map.get(list_type)} {list_level}"
+                new_par = self.Report.add_paragraph(style=par_style)
+                # Add logic for resetting a paragraph numbering for ordered list
+                for elem in lst:
+                    if num_res_help_ol.get(elem.get('list_level')) is None and elem.get('symbol') == 'ol':
+                        num_res_help_ol[elem.get('list_level')] = elem.get('symbol_id')
+                    elif num_res_help_ol.get(elem.get('list_level')) == elem.get('symbol_id') and elem.get(
+                            'symbol') == 'ol':
+                        pass
+                    elif num_res_help_ol.get(elem.get('list_level')) != elem.get('symbol_id') and elem.get(
+                            'symbol') == 'ol':
+                        num_res_help_ol[elem.get('list_level')] = elem.get('symbol_id')
+                        self.reset_paragraph_numbering(document=self.Report, paragraph=new_par)
+                # Add logic for resetting a paragraph numbering for unordered list
+                for elem in lst:
+                    if num_res_help_ul.get(elem.get('list_level')) is None and elem.get('symbol') == 'ul':
+                        num_res_help_ul[elem.get('list_level')] = elem.get('symbol_id')
+                    elif num_res_help_ul.get(elem.get('list_level')) == elem.get('symbol_id') and elem.get(
+                            'symbol') == 'ul':
+                        pass
+                    elif num_res_help_ul.get(elem.get('list_level')) != elem.get('symbol_id') and elem.get(
+                            'symbol') == 'ul':
+                        num_res_help_ul[elem.get('list_level')] = elem.get('symbol_id')
+                        self.reset_paragraph_numbering(document=self.Report, paragraph=new_par)
+            else:
                 new_par = self.Report.add_paragraph()
-                algn_map = {'CENTER': WD_ALIGN_PARAGRAPH.CENTER, 'LEFT': WD_ALIGN_PARAGRAPH.LEFT,
-                            'RIGHT': WD_ALIGN_PARAGRAPH.RIGHT, 'JUSTIFY': WD_ALIGN_PARAGRAPH.JUSTIFY}
-                new_par.alignment = algn_map.get(amfsh[0], WD_ALIGN_PARAGRAPH.LEFT)
-                if amfsh[2] is not None and amfsh[2][0] == 'margin-right':
-                    new_par.paragraph_format.right_indent = Pt(amfsh[2][1])
-                elif amfsh[2] is not None and amfsh[2][0] == 'margin-left':
-                    new_par.paragraph_format.left_indent = Pt(amfsh[2][1])
-                self.generate_multi_font_string(multi_font_string=amfsh[1], paragraph=new_par)
-            # Checking for list. List instance is dedicated for ordered and unordered lists (needs paragraph styling)
-            elif isinstance(amfsh, list):
-                # amfsh as list: [un/ordered list element, un/ordered list element, un/ordered list element, ...]
-                for elem in amfsh:
-                    # elem as list: [alignment, argMultiFontStringHelper, margin, list style]
-                    new_par = self.Report.add_paragraph(style=elem[3])
-                    algn_map = {'CENTER': WD_ALIGN_PARAGRAPH.CENTER, 'LEFT': WD_ALIGN_PARAGRAPH.LEFT,
-                                'RIGHT': WD_ALIGN_PARAGRAPH.RIGHT, 'JUSTIFY': WD_ALIGN_PARAGRAPH.JUSTIFY}
-                    new_par.alignment = algn_map.get(elem[0], WD_ALIGN_PARAGRAPH.LEFT)
-                    if elem[2] is not None and elem[2][0] == 'margin-right':
-                        new_par.paragraph_format.right_indent = Pt(elem[2][1])
-                    elif elem[2] is not None and elem[2][0] == 'margin-left':
-                        new_par.paragraph_format.left_indent = Pt(elem[2][1])
-                    self.generate_multi_font_string(multi_font_string=elem[1], paragraph=new_par)
+            new_par.alignment = alingment_map.get(amfsh_dict['alignment'].get(uuid_key), WD_ALIGN_PARAGRAPH.LEFT)
+            if (indent := amfsh_dict['indent'].get(uuid_key)) is not None and indent[0] == 'margin-right':
+                new_par.paragraph_format.right_indent = Pt(indent[1])
+            elif indent is not None and indent[0] == 'margin-left':
+                new_par.paragraph_format.left_indent = Pt(indent[1])
+            self.generate_multi_font_string(multi_font_string=amfsh, paragraph=new_par)
+
+    @staticmethod
+    def reset_paragraph_numbering(document, paragraph):
+        """ Look up the associated level and numbering style associated with the paragraph's style.
+            Insert a new numbering definition with an override, then associate it to the paragraph.
+        """
+        styp = docx.enum.style.WD_STYLE_TYPE
+        assoc_style = document.styles.get_by_id(paragraph._element.pPr.pStyle.val, styp.PARAGRAPH).element.pPr.numPr
+        try:
+            assoc_level = assoc_style.ilvl.val
+        except:
+            assoc_level = 0
+        assoc_num_id = assoc_style.numId.val
+        assoc_abstract_num_id = document.part.numbering_part.element.num_having_numId(assoc_num_id).abstractNumId.val
+        new_num = document.part.numbering_part.element.add_num(assoc_abstract_num_id)
+        new_num.add_lvlOverride(assoc_level).get_or_add_startOverride().val = 1
+        nn = paragraph._element.pPr.get_or_add_numPr()
+        nn.get_or_add_ilvl().val = assoc_level
+        nn.get_or_add_numId().val = new_num.numId
+
 
     @staticmethod
     def insert_paragraph_after(paragraph, text=None, style=None):
